@@ -1,4 +1,5 @@
 """Time offset classes for use with cftime.datetime objects"""
+
 # The offset classes and mechanisms for generating time ranges defined in
 # this module were copied/adapted from those defined in pandas.  See in
 # particular the objects and methods defined in pandas.tseries.offsets
@@ -42,12 +43,14 @@
 from __future__ import annotations
 
 import re
+from collections.abc import Mapping
 from datetime import datetime, timedelta
 from functools import partial
-from typing import TYPE_CHECKING, ClassVar
+from typing import TYPE_CHECKING, ClassVar, Literal
 
 import numpy as np
 import pandas as pd
+from packaging.version import Version
 
 from xarray.coding.cftimeindex import CFTimeIndex, _parse_iso8601_with_reso
 from xarray.coding.times import (
@@ -72,7 +75,18 @@ except ImportError:
 
 
 if TYPE_CHECKING:
-    from xarray.core.types import InclusiveOptions, SideOptions
+    from xarray.core.types import InclusiveOptions, Self, SideOptions, TypeAlias
+
+
+DayOption: TypeAlias = Literal["start", "end"]
+
+
+def _nanosecond_precision_timestamp(*args, **kwargs):
+    # As of pandas version 3.0, pd.to_datetime(Timestamp(...)) will try to
+    # infer the appropriate datetime precision. Until xarray supports
+    # non-nanosecond precision times, we will use this constructor wrapper to
+    # explicitly create nanosecond-precision Timestamp objects.
+    return pd.Timestamp(*args, **kwargs).as_unit("ns")
 
 
 def get_date_type(calendar, use_cftime=True):
@@ -81,7 +95,7 @@ def get_date_type(calendar, use_cftime=True):
         raise ImportError("cftime is required for dates with non-standard calendars")
     else:
         if _is_standard_calendar(calendar) and not use_cftime:
-            return pd.Timestamp
+            return _nanosecond_precision_timestamp
 
         calendars = {
             "noleap": cftime.DatetimeNoLeap,
@@ -99,9 +113,10 @@ def get_date_type(calendar, use_cftime=True):
 
 class BaseCFTimeOffset:
     _freq: ClassVar[str | None] = None
-    _day_option: ClassVar[str | None] = None
+    _day_option: ClassVar[DayOption | None] = None
+    n: int
 
-    def __init__(self, n: int = 1):
+    def __init__(self, n: int = 1) -> None:
         if not isinstance(n, int):
             raise TypeError(
                 "The provided multiple 'n' must be an integer. "
@@ -109,13 +124,15 @@ class BaseCFTimeOffset:
             )
         self.n = n
 
-    def rule_code(self):
+    def rule_code(self) -> str | None:
         return self._freq
 
-    def __eq__(self, other):
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, BaseCFTimeOffset):
+            return NotImplemented
         return self.n == other.n and self.rule_code() == other.rule_code()
 
-    def __ne__(self, other):
+    def __ne__(self, other: object) -> bool:
         return not self == other
 
     def __add__(self, other):
@@ -132,12 +149,12 @@ class BaseCFTimeOffset:
         else:
             return NotImplemented
 
-    def __mul__(self, other):
+    def __mul__(self, other: int) -> Self:
         if not isinstance(other, int):
             return NotImplemented
         return type(self)(n=other * self.n)
 
-    def __neg__(self):
+    def __neg__(self) -> Self:
         return self * -1
 
     def __rmul__(self, other):
@@ -151,10 +168,10 @@ class BaseCFTimeOffset:
             raise TypeError("Cannot subtract cftime offsets of differing types")
         return -self + other
 
-    def __apply__(self):
+    def __apply__(self, other):
         return NotImplemented
 
-    def onOffset(self, date):
+    def onOffset(self, date) -> bool:
         """Check if the given date is in the set of possible dates created
         using a length-one version of this offset class."""
         test_date = (self + date) - self
@@ -187,22 +204,21 @@ class BaseCFTimeOffset:
 class Tick(BaseCFTimeOffset):
     # analogous https://github.com/pandas-dev/pandas/blob/ccb25ab1d24c4fb9691270706a59c8d319750870/pandas/_libs/tslibs/offsets.pyx#L806
 
-    def _next_higher_resolution(self):
+    def _next_higher_resolution(self) -> Tick:
         self_type = type(self)
-        if self_type not in [Day, Hour, Minute, Second, Millisecond]:
-            raise ValueError("Could not convert to integer offset at any resolution")
-        if type(self) is Day:
+        if self_type is Day:
             return Hour(self.n * 24)
-        if type(self) is Hour:
+        if self_type is Hour:
             return Minute(self.n * 60)
-        if type(self) is Minute:
+        if self_type is Minute:
             return Second(self.n * 60)
-        if type(self) is Second:
+        if self_type is Second:
             return Millisecond(self.n * 1000)
-        if type(self) is Millisecond:
+        if self_type is Millisecond:
             return Microsecond(self.n * 1000)
+        raise ValueError("Could not convert to integer offset at any resolution")
 
-    def __mul__(self, other):
+    def __mul__(self, other: int | float) -> Tick:
         if not isinstance(other, (int, float)):
             return NotImplemented
         if isinstance(other, float):
@@ -217,12 +233,12 @@ class Tick(BaseCFTimeOffset):
             return new_self * other
         return type(self)(n=other * self.n)
 
-    def as_timedelta(self):
+    def as_timedelta(self) -> timedelta:
         """All Tick subclasses must implement an as_timedelta method."""
         raise NotImplementedError
 
 
-def _get_day_of_month(other, day_option):
+def _get_day_of_month(other, day_option: DayOption) -> int:
     """Find the day in `other`'s month that satisfies a BaseCFTimeOffset's
     onOffset policy, as described by the `day_option` argument.
 
@@ -241,14 +257,13 @@ def _get_day_of_month(other, day_option):
 
     if day_option == "start":
         return 1
-    elif day_option == "end":
+    if day_option == "end":
         return _days_in_month(other)
-    elif day_option is None:
+    if day_option is None:
         # Note: unlike `_shift_month`, _get_day_of_month does not
         # allow day_option = None
         raise NotImplementedError()
-    else:
-        raise ValueError(day_option)
+    raise ValueError(day_option)
 
 
 def _days_in_month(date):
@@ -283,7 +298,7 @@ def _adjust_n_years(other, n, month, reference_day):
     return n
 
 
-def _shift_month(date, months, day_option="start"):
+def _shift_month(date, months, day_option: DayOption = "start"):
     """Shift the date to a month start or end a given number of months away."""
     if cftime is None:
         raise ModuleNotFoundError("No module named 'cftime'")
@@ -306,7 +321,9 @@ def _shift_month(date, months, day_option="start"):
     return date.replace(year=year, month=month, day=day)
 
 
-def roll_qtrday(other, n, month, day_option, modby=3):
+def roll_qtrday(
+    other, n: int, month: int, day_option: DayOption, modby: int = 3
+) -> int:
     """Possibly increment or decrement the number of periods to shift
     based on rollforward/rollbackward conventions.
 
@@ -347,7 +364,7 @@ def roll_qtrday(other, n, month, day_option, modby=3):
     return n
 
 
-def _validate_month(month, default_month):
+def _validate_month(month: int | None, default_month: int) -> int:
     result_month = default_month if month is None else month
     if not isinstance(result_month, int):
         raise TypeError(
@@ -371,20 +388,20 @@ class MonthBegin(BaseCFTimeOffset):
         n = _adjust_n_months(other.day, self.n, 1)
         return _shift_month(other, n, "start")
 
-    def onOffset(self, date):
+    def onOffset(self, date) -> bool:
         """Check if the given date is in the set of possible dates created
         using a length-one version of this offset class."""
         return date.day == 1
 
 
 class MonthEnd(BaseCFTimeOffset):
-    _freq = "M"
+    _freq = "ME"
 
     def __apply__(self, other):
         n = _adjust_n_months(other.day, self.n, _days_in_month(other))
         return _shift_month(other, n, "end")
 
-    def onOffset(self, date):
+    def onOffset(self, date) -> bool:
         """Check if the given date is in the set of possible dates created
         using a length-one version of this offset class."""
         return date.day == _days_in_month(date)
@@ -409,10 +426,10 @@ _MONTH_ABBREVIATIONS = {
 class QuarterOffset(BaseCFTimeOffset):
     """Quarter representation copied off of pandas/tseries/offsets.py"""
 
-    _freq: ClassVar[str]
     _default_month: ClassVar[int]
+    month: int
 
-    def __init__(self, n=1, month=None):
+    def __init__(self, n: int = 1, month: int | None = None) -> None:
         BaseCFTimeOffset.__init__(self, n)
         self.month = _validate_month(month, self._default_month)
 
@@ -429,29 +446,28 @@ class QuarterOffset(BaseCFTimeOffset):
         months = qtrs * 3 - months_since
         return _shift_month(other, months, self._day_option)
 
-    def onOffset(self, date):
+    def onOffset(self, date) -> bool:
         """Check if the given date is in the set of possible dates created
         using a length-one version of this offset class."""
         mod_month = (date.month - self.month) % 3
         return mod_month == 0 and date.day == self._get_offset_day(date)
 
-    def __sub__(self, other):
+    def __sub__(self, other: Self) -> Self:
         if cftime is None:
             raise ModuleNotFoundError("No module named 'cftime'")
 
         if isinstance(other, cftime.datetime):
             raise TypeError("Cannot subtract cftime.datetime from offset.")
-        elif type(other) == type(self) and other.month == self.month:
+        if type(other) == type(self) and other.month == self.month:
             return type(self)(self.n - other.n, month=self.month)
-        else:
-            return NotImplemented
+        return NotImplemented
 
     def __mul__(self, other):
         if isinstance(other, float):
             return NotImplemented
         return type(self)(n=other * self.n, month=self.month)
 
-    def rule_code(self):
+    def rule_code(self) -> str:
         return f"{self._freq}-{_MONTH_ABBREVIATIONS[self.month]}"
 
     def __str__(self):
@@ -490,7 +506,7 @@ class QuarterEnd(QuarterOffset):
     # from the constructor, however, the default month is March.
     # We follow that behavior here.
     _default_month = 3
-    _freq = "Q"
+    _freq = "QE"
     _day_option = "end"
 
     def rollforward(self, date):
@@ -509,11 +525,10 @@ class QuarterEnd(QuarterOffset):
 
 
 class YearOffset(BaseCFTimeOffset):
-    _freq: ClassVar[str]
-    _day_option: ClassVar[str]
     _default_month: ClassVar[int]
+    month: int
 
-    def __init__(self, n=1, month=None):
+    def __init__(self, n: int = 1, month: int | None = None) -> None:
         BaseCFTimeOffset.__init__(self, n)
         self.month = _validate_month(month, self._default_month)
 
@@ -539,19 +554,19 @@ class YearOffset(BaseCFTimeOffset):
             return NotImplemented
         return type(self)(n=other * self.n, month=self.month)
 
-    def rule_code(self):
+    def rule_code(self) -> str:
         return f"{self._freq}-{_MONTH_ABBREVIATIONS[self.month]}"
 
-    def __str__(self):
+    def __str__(self) -> str:
         return f"<{type(self).__name__}: n={self.n}, month={self.month}>"
 
 
 class YearBegin(YearOffset):
-    _freq = "AS"
+    _freq = "YS"
     _day_option = "start"
     _default_month = 1
 
-    def onOffset(self, date):
+    def onOffset(self, date) -> bool:
         """Check if the given date is in the set of possible dates created
         using a length-one version of this offset class."""
         return date.day == 1 and date.month == self.month
@@ -572,11 +587,11 @@ class YearBegin(YearOffset):
 
 
 class YearEnd(YearOffset):
-    _freq = "A"
+    _freq = "YE"
     _day_option = "end"
     _default_month = 12
 
-    def onOffset(self, date):
+    def onOffset(self, date) -> bool:
         """Check if the given date is in the set of possible dates created
         using a length-one version of this offset class."""
         return date.day == _days_in_month(date) and date.month == self.month
@@ -599,7 +614,7 @@ class YearEnd(YearOffset):
 class Day(Tick):
     _freq = "D"
 
-    def as_timedelta(self):
+    def as_timedelta(self) -> timedelta:
         return timedelta(days=self.n)
 
     def __apply__(self, other):
@@ -607,9 +622,9 @@ class Day(Tick):
 
 
 class Hour(Tick):
-    _freq = "H"
+    _freq = "h"
 
-    def as_timedelta(self):
+    def as_timedelta(self) -> timedelta:
         return timedelta(hours=self.n)
 
     def __apply__(self, other):
@@ -617,9 +632,9 @@ class Hour(Tick):
 
 
 class Minute(Tick):
-    _freq = "T"
+    _freq = "min"
 
-    def as_timedelta(self):
+    def as_timedelta(self) -> timedelta:
         return timedelta(minutes=self.n)
 
     def __apply__(self, other):
@@ -627,9 +642,9 @@ class Minute(Tick):
 
 
 class Second(Tick):
-    _freq = "S"
+    _freq = "s"
 
-    def as_timedelta(self):
+    def as_timedelta(self) -> timedelta:
         return timedelta(seconds=self.n)
 
     def __apply__(self, other):
@@ -637,9 +652,9 @@ class Second(Tick):
 
 
 class Millisecond(Tick):
-    _freq = "L"
+    _freq = "ms"
 
-    def as_timedelta(self):
+    def as_timedelta(self) -> timedelta:
         return timedelta(milliseconds=self.n)
 
     def __apply__(self, other):
@@ -647,86 +662,61 @@ class Millisecond(Tick):
 
 
 class Microsecond(Tick):
-    _freq = "U"
+    _freq = "us"
 
-    def as_timedelta(self):
+    def as_timedelta(self) -> timedelta:
         return timedelta(microseconds=self.n)
 
     def __apply__(self, other):
         return other + self.as_timedelta()
 
 
-_FREQUENCIES = {
+def _generate_anchored_offsets(
+    base_freq: str, offset: type[YearOffset | QuarterOffset]
+) -> dict[str, type[BaseCFTimeOffset]]:
+    offsets: dict[str, type[BaseCFTimeOffset]] = {}
+    for month, abbreviation in _MONTH_ABBREVIATIONS.items():
+        anchored_freq = f"{base_freq}-{abbreviation}"
+        offsets[anchored_freq] = partial(offset, month=month)  # type: ignore[assignment]
+    return offsets
+
+
+_FREQUENCIES: Mapping[str, type[BaseCFTimeOffset]] = {
     "A": YearEnd,
     "AS": YearBegin,
     "Y": YearEnd,
+    "YE": YearEnd,
     "YS": YearBegin,
-    "Q": partial(QuarterEnd, month=12),
-    "QS": partial(QuarterBegin, month=1),
+    "Q": partial(QuarterEnd, month=12),  # type: ignore[dict-item]
+    "QE": partial(QuarterEnd, month=12),  # type: ignore[dict-item]
+    "QS": partial(QuarterBegin, month=1),  # type: ignore[dict-item]
     "M": MonthEnd,
+    "ME": MonthEnd,
     "MS": MonthBegin,
     "D": Day,
     "H": Hour,
+    "h": Hour,
     "T": Minute,
     "min": Minute,
     "S": Second,
+    "s": Second,
     "L": Millisecond,
     "ms": Millisecond,
     "U": Microsecond,
     "us": Microsecond,
-    "AS-JAN": partial(YearBegin, month=1),
-    "AS-FEB": partial(YearBegin, month=2),
-    "AS-MAR": partial(YearBegin, month=3),
-    "AS-APR": partial(YearBegin, month=4),
-    "AS-MAY": partial(YearBegin, month=5),
-    "AS-JUN": partial(YearBegin, month=6),
-    "AS-JUL": partial(YearBegin, month=7),
-    "AS-AUG": partial(YearBegin, month=8),
-    "AS-SEP": partial(YearBegin, month=9),
-    "AS-OCT": partial(YearBegin, month=10),
-    "AS-NOV": partial(YearBegin, month=11),
-    "AS-DEC": partial(YearBegin, month=12),
-    "A-JAN": partial(YearEnd, month=1),
-    "A-FEB": partial(YearEnd, month=2),
-    "A-MAR": partial(YearEnd, month=3),
-    "A-APR": partial(YearEnd, month=4),
-    "A-MAY": partial(YearEnd, month=5),
-    "A-JUN": partial(YearEnd, month=6),
-    "A-JUL": partial(YearEnd, month=7),
-    "A-AUG": partial(YearEnd, month=8),
-    "A-SEP": partial(YearEnd, month=9),
-    "A-OCT": partial(YearEnd, month=10),
-    "A-NOV": partial(YearEnd, month=11),
-    "A-DEC": partial(YearEnd, month=12),
-    "QS-JAN": partial(QuarterBegin, month=1),
-    "QS-FEB": partial(QuarterBegin, month=2),
-    "QS-MAR": partial(QuarterBegin, month=3),
-    "QS-APR": partial(QuarterBegin, month=4),
-    "QS-MAY": partial(QuarterBegin, month=5),
-    "QS-JUN": partial(QuarterBegin, month=6),
-    "QS-JUL": partial(QuarterBegin, month=7),
-    "QS-AUG": partial(QuarterBegin, month=8),
-    "QS-SEP": partial(QuarterBegin, month=9),
-    "QS-OCT": partial(QuarterBegin, month=10),
-    "QS-NOV": partial(QuarterBegin, month=11),
-    "QS-DEC": partial(QuarterBegin, month=12),
-    "Q-JAN": partial(QuarterEnd, month=1),
-    "Q-FEB": partial(QuarterEnd, month=2),
-    "Q-MAR": partial(QuarterEnd, month=3),
-    "Q-APR": partial(QuarterEnd, month=4),
-    "Q-MAY": partial(QuarterEnd, month=5),
-    "Q-JUN": partial(QuarterEnd, month=6),
-    "Q-JUL": partial(QuarterEnd, month=7),
-    "Q-AUG": partial(QuarterEnd, month=8),
-    "Q-SEP": partial(QuarterEnd, month=9),
-    "Q-OCT": partial(QuarterEnd, month=10),
-    "Q-NOV": partial(QuarterEnd, month=11),
-    "Q-DEC": partial(QuarterEnd, month=12),
+    **_generate_anchored_offsets("AS", YearBegin),
+    **_generate_anchored_offsets("A", YearEnd),
+    **_generate_anchored_offsets("YS", YearBegin),
+    **_generate_anchored_offsets("Y", YearEnd),
+    **_generate_anchored_offsets("YE", YearEnd),
+    **_generate_anchored_offsets("QS", QuarterBegin),
+    **_generate_anchored_offsets("Q", QuarterEnd),
+    **_generate_anchored_offsets("QE", QuarterEnd),
 }
 
 
 _FREQUENCY_CONDITION = "|".join(_FREQUENCIES.keys())
-_PATTERN = rf"^((?P<multiple>\d+)|())(?P<freq>({_FREQUENCY_CONDITION}))$"
+_PATTERN = rf"^((?P<multiple>[+-]?\d+)|())(?P<freq>({_FREQUENCY_CONDITION}))$"
 
 
 # pandas defines these offsets as "Tick" objects, which for instance have
@@ -734,18 +724,64 @@ _PATTERN = rf"^((?P<multiple>\d+)|())(?P<freq>({_FREQUENCY_CONDITION}))$"
 CFTIME_TICKS = (Day, Hour, Minute, Second)
 
 
-def to_offset(freq):
+def _generate_anchored_deprecated_frequencies(
+    deprecated: str, recommended: str
+) -> dict[str, str]:
+    pairs = {}
+    for abbreviation in _MONTH_ABBREVIATIONS.values():
+        anchored_deprecated = f"{deprecated}-{abbreviation}"
+        anchored_recommended = f"{recommended}-{abbreviation}"
+        pairs[anchored_deprecated] = anchored_recommended
+    return pairs
+
+
+_DEPRECATED_FREQUENICES: dict[str, str] = {
+    "A": "YE",
+    "Y": "YE",
+    "AS": "YS",
+    "Q": "QE",
+    "M": "ME",
+    "H": "h",
+    "T": "min",
+    "S": "s",
+    "L": "ms",
+    "U": "us",
+    **_generate_anchored_deprecated_frequencies("A", "YE"),
+    **_generate_anchored_deprecated_frequencies("Y", "YE"),
+    **_generate_anchored_deprecated_frequencies("AS", "YS"),
+    **_generate_anchored_deprecated_frequencies("Q", "QE"),
+}
+
+
+_DEPRECATION_MESSAGE = (
+    "{deprecated_freq!r} is deprecated and will be removed in a future "
+    "version. Please use {recommended_freq!r} instead of "
+    "{deprecated_freq!r}."
+)
+
+
+def _emit_freq_deprecation_warning(deprecated_freq):
+    recommended_freq = _DEPRECATED_FREQUENICES[deprecated_freq]
+    message = _DEPRECATION_MESSAGE.format(
+        deprecated_freq=deprecated_freq, recommended_freq=recommended_freq
+    )
+    emit_user_level_warning(message, FutureWarning)
+
+
+def to_offset(freq: BaseCFTimeOffset | str, warn: bool = True) -> BaseCFTimeOffset:
     """Convert a frequency string to the appropriate subclass of
     BaseCFTimeOffset."""
     if isinstance(freq, BaseCFTimeOffset):
         return freq
-    else:
-        try:
-            freq_data = re.match(_PATTERN, freq).groupdict()
-        except AttributeError:
-            raise ValueError("Invalid frequency string provided")
+
+    match = re.match(_PATTERN, freq)
+    if match is None:
+        raise ValueError("Invalid frequency string provided")
+    freq_data = match.groupdict()
 
     freq = freq_data["freq"]
+    if warn and freq in _DEPRECATED_FREQUENICES:
+        _emit_freq_deprecation_warning(freq)
     multiples = freq_data["multiple"]
     multiples = 1 if multiples is None else int(multiples)
     return _FREQUENCIES[freq](n=multiples)
@@ -807,7 +843,8 @@ def _generate_range(start, end, periods, offset):
     """Generate a regular range of cftime.datetime objects with a
     given time offset.
 
-    Adapted from pandas.tseries.offsets.generate_range.
+    Adapted from pandas.tseries.offsets.generate_range (now at
+    pandas.core.arrays.datetimes._generate_range).
 
     Parameters
     ----------
@@ -825,12 +862,14 @@ def _generate_range(start, end, periods, offset):
     A generator object
     """
     if start:
-        start = offset.rollforward(start)
+        # From pandas GH 56147 / 56832 to account for negative direction and
+        # range bounds
+        if offset.n >= 0:
+            start = offset.rollforward(start)
+        else:
+            start = offset.rollback(start)
 
-    if end:
-        end = offset.rollback(end)
-
-    if periods is None and end < start:
+    if periods is None and end < start and offset.n >= 0:
         end = None
         periods = 0
 
@@ -879,7 +918,9 @@ def _translate_closed_to_inclusive(closed):
     return inclusive
 
 
-def _infer_inclusive(closed, inclusive):
+def _infer_inclusive(
+    closed: NoDefault | SideOptions, inclusive: InclusiveOptions | None
+) -> InclusiveOptions:
     """Follows code added in pandas #43504."""
     if closed is not no_default and inclusive is not None:
         raise ValueError(
@@ -887,9 +928,9 @@ def _infer_inclusive(closed, inclusive):
             "passed if argument `inclusive` is not None."
         )
     if closed is not no_default:
-        inclusive = _translate_closed_to_inclusive(closed)
-    elif inclusive is None:
-        inclusive = "both"
+        return _translate_closed_to_inclusive(closed)
+    if inclusive is None:
+        return "both"
     return inclusive
 
 
@@ -897,13 +938,13 @@ def cftime_range(
     start=None,
     end=None,
     periods=None,
-    freq="D",
+    freq=None,
     normalize=False,
     name=None,
     closed: NoDefault | SideOptions = no_default,
     inclusive: None | InclusiveOptions = None,
     calendar="standard",
-):
+) -> CFTimeIndex:
     """Return a fixed frequency CFTimeIndex.
 
     Parameters
@@ -915,7 +956,7 @@ def cftime_range(
     periods : int, optional
         Number of periods to generate.
     freq : str or None, default: "D"
-        Frequency strings can have multiples, e.g. "5H".
+        Frequency strings can have multiples, e.g. "5h" and negative values, e.g. "-1D".
     normalize : bool, default: False
         Normalize start/end dates to midnight before generating date range.
     name : str, default: None
@@ -965,84 +1006,84 @@ def cftime_range(
     +--------+--------------------------+
     | Alias  | Description              |
     +========+==========================+
-    | A, Y   | Year-end frequency       |
+    | YE     | Year-end frequency       |
     +--------+--------------------------+
-    | AS, YS | Year-start frequency     |
+    | YS     | Year-start frequency     |
     +--------+--------------------------+
-    | Q      | Quarter-end frequency    |
+    | QE     | Quarter-end frequency    |
     +--------+--------------------------+
     | QS     | Quarter-start frequency  |
     +--------+--------------------------+
-    | M      | Month-end frequency      |
+    | ME     | Month-end frequency      |
     +--------+--------------------------+
     | MS     | Month-start frequency    |
     +--------+--------------------------+
     | D      | Day frequency            |
     +--------+--------------------------+
-    | H      | Hour frequency           |
+    | h      | Hour frequency           |
     +--------+--------------------------+
-    | T, min | Minute frequency         |
+    | min    | Minute frequency         |
     +--------+--------------------------+
-    | S      | Second frequency         |
+    | s      | Second frequency         |
     +--------+--------------------------+
-    | L, ms  | Millisecond frequency    |
+    | ms     | Millisecond frequency    |
     +--------+--------------------------+
-    | U, us  | Microsecond frequency    |
+    | us     | Microsecond frequency    |
     +--------+--------------------------+
 
     Any multiples of the following anchored offsets are also supported.
 
-    +----------+--------------------------------------------------------------------+
-    | Alias    | Description                                                        |
-    +==========+====================================================================+
-    | A(S)-JAN | Annual frequency, anchored at the end (or beginning) of January    |
-    +----------+--------------------------------------------------------------------+
-    | A(S)-FEB | Annual frequency, anchored at the end (or beginning) of February   |
-    +----------+--------------------------------------------------------------------+
-    | A(S)-MAR | Annual frequency, anchored at the end (or beginning) of March      |
-    +----------+--------------------------------------------------------------------+
-    | A(S)-APR | Annual frequency, anchored at the end (or beginning) of April      |
-    +----------+--------------------------------------------------------------------+
-    | A(S)-MAY | Annual frequency, anchored at the end (or beginning) of May        |
-    +----------+--------------------------------------------------------------------+
-    | A(S)-JUN | Annual frequency, anchored at the end (or beginning) of June       |
-    +----------+--------------------------------------------------------------------+
-    | A(S)-JUL | Annual frequency, anchored at the end (or beginning) of July       |
-    +----------+--------------------------------------------------------------------+
-    | A(S)-AUG | Annual frequency, anchored at the end (or beginning) of August     |
-    +----------+--------------------------------------------------------------------+
-    | A(S)-SEP | Annual frequency, anchored at the end (or beginning) of September  |
-    +----------+--------------------------------------------------------------------+
-    | A(S)-OCT | Annual frequency, anchored at the end (or beginning) of October    |
-    +----------+--------------------------------------------------------------------+
-    | A(S)-NOV | Annual frequency, anchored at the end (or beginning) of November   |
-    +----------+--------------------------------------------------------------------+
-    | A(S)-DEC | Annual frequency, anchored at the end (or beginning) of December   |
-    +----------+--------------------------------------------------------------------+
-    | Q(S)-JAN | Quarter frequency, anchored at the end (or beginning) of January   |
-    +----------+--------------------------------------------------------------------+
-    | Q(S)-FEB | Quarter frequency, anchored at the end (or beginning) of February  |
-    +----------+--------------------------------------------------------------------+
-    | Q(S)-MAR | Quarter frequency, anchored at the end (or beginning) of March     |
-    +----------+--------------------------------------------------------------------+
-    | Q(S)-APR | Quarter frequency, anchored at the end (or beginning) of April     |
-    +----------+--------------------------------------------------------------------+
-    | Q(S)-MAY | Quarter frequency, anchored at the end (or beginning) of May       |
-    +----------+--------------------------------------------------------------------+
-    | Q(S)-JUN | Quarter frequency, anchored at the end (or beginning) of June      |
-    +----------+--------------------------------------------------------------------+
-    | Q(S)-JUL | Quarter frequency, anchored at the end (or beginning) of July      |
-    +----------+--------------------------------------------------------------------+
-    | Q(S)-AUG | Quarter frequency, anchored at the end (or beginning) of August    |
-    +----------+--------------------------------------------------------------------+
-    | Q(S)-SEP | Quarter frequency, anchored at the end (or beginning) of September |
-    +----------+--------------------------------------------------------------------+
-    | Q(S)-OCT | Quarter frequency, anchored at the end (or beginning) of October   |
-    +----------+--------------------------------------------------------------------+
-    | Q(S)-NOV | Quarter frequency, anchored at the end (or beginning) of November  |
-    +----------+--------------------------------------------------------------------+
-    | Q(S)-DEC | Quarter frequency, anchored at the end (or beginning) of December  |
-    +----------+--------------------------------------------------------------------+
+    +------------+--------------------------------------------------------------------+
+    | Alias      | Description                                                        |
+    +============+====================================================================+
+    | Y(E,S)-JAN | Annual frequency, anchored at the (end, beginning) of January      |
+    +------------+--------------------------------------------------------------------+
+    | Y(E,S)-FEB | Annual frequency, anchored at the (end, beginning) of February     |
+    +------------+--------------------------------------------------------------------+
+    | Y(E,S)-MAR | Annual frequency, anchored at the (end, beginning) of March        |
+    +------------+--------------------------------------------------------------------+
+    | Y(E,S)-APR | Annual frequency, anchored at the (end, beginning) of April        |
+    +------------+--------------------------------------------------------------------+
+    | Y(E,S)-MAY | Annual frequency, anchored at the (end, beginning) of May          |
+    +------------+--------------------------------------------------------------------+
+    | Y(E,S)-JUN | Annual frequency, anchored at the (end, beginning) of June         |
+    +------------+--------------------------------------------------------------------+
+    | Y(E,S)-JUL | Annual frequency, anchored at the (end, beginning) of July         |
+    +------------+--------------------------------------------------------------------+
+    | Y(E,S)-AUG | Annual frequency, anchored at the (end, beginning) of August       |
+    +------------+--------------------------------------------------------------------+
+    | Y(E,S)-SEP | Annual frequency, anchored at the (end, beginning) of September    |
+    +------------+--------------------------------------------------------------------+
+    | Y(E,S)-OCT | Annual frequency, anchored at the (end, beginning) of October      |
+    +------------+--------------------------------------------------------------------+
+    | Y(E,S)-NOV | Annual frequency, anchored at the (end, beginning) of November     |
+    +------------+--------------------------------------------------------------------+
+    | Y(E,S)-DEC | Annual frequency, anchored at the (end, beginning) of December     |
+    +------------+--------------------------------------------------------------------+
+    | Q(E,S)-JAN | Quarter frequency, anchored at the (end, beginning) of January     |
+    +------------+--------------------------------------------------------------------+
+    | Q(E,S)-FEB | Quarter frequency, anchored at the (end, beginning) of February    |
+    +------------+--------------------------------------------------------------------+
+    | Q(E,S)-MAR | Quarter frequency, anchored at the (end, beginning) of March       |
+    +------------+--------------------------------------------------------------------+
+    | Q(E,S)-APR | Quarter frequency, anchored at the (end, beginning) of April       |
+    +------------+--------------------------------------------------------------------+
+    | Q(E,S)-MAY | Quarter frequency, anchored at the (end, beginning) of May         |
+    +------------+--------------------------------------------------------------------+
+    | Q(E,S)-JUN | Quarter frequency, anchored at the (end, beginning) of June        |
+    +------------+--------------------------------------------------------------------+
+    | Q(E,S)-JUL | Quarter frequency, anchored at the (end, beginning) of July        |
+    +------------+--------------------------------------------------------------------+
+    | Q(E,S)-AUG | Quarter frequency, anchored at the (end, beginning) of August      |
+    +------------+--------------------------------------------------------------------+
+    | Q(E,S)-SEP | Quarter frequency, anchored at the (end, beginning) of September   |
+    +------------+--------------------------------------------------------------------+
+    | Q(E,S)-OCT | Quarter frequency, anchored at the (end, beginning) of October     |
+    +------------+--------------------------------------------------------------------+
+    | Q(E,S)-NOV | Quarter frequency, anchored at the (end, beginning) of November    |
+    +------------+--------------------------------------------------------------------+
+    | Q(E,S)-DEC | Quarter frequency, anchored at the (end, beginning) of December    |
+    +------------+--------------------------------------------------------------------+
 
     Finally, the following calendar aliases are supported.
 
@@ -1083,6 +1124,10 @@ def cftime_range(
     --------
     pandas.date_range
     """
+
+    if freq is None and any(arg is None for arg in [periods, start, end]):
+        freq = "D"
+
     # Adapted from pandas.core.indexes.datetimes._generate_range.
     if count_not_none(start, end, periods, freq) != 3:
         raise ValueError(
@@ -1135,7 +1180,7 @@ def date_range(
     start=None,
     end=None,
     periods=None,
-    freq="D",
+    freq=None,
     tz=None,
     normalize=False,
     name=None,
@@ -1158,7 +1203,7 @@ def date_range(
     periods : int, optional
         Number of periods to generate.
     freq : str or None, default: "D"
-        Frequency strings can have multiples, e.g. "5H".
+        Frequency strings can have multiples, e.g. "5h" and negative values, e.g. "-1D".
     tz : str or tzinfo, optional
         Time zone name for returning localized DatetimeIndex, for example
         'Asia/Hong_Kong'. By default, the resulting DatetimeIndex is
@@ -1212,7 +1257,8 @@ def date_range(
                 start=start,
                 end=end,
                 periods=periods,
-                freq=freq,
+                # TODO remove translation once requiring pandas >= 2.2
+                freq=_new_to_legacy_freq(freq),
                 tz=tz,
                 normalize=normalize,
                 name=name,
@@ -1240,6 +1286,96 @@ def date_range(
     )
 
 
+def _new_to_legacy_freq(freq):
+    # xarray will now always return "ME" and "QE" for MonthEnd and QuarterEnd
+    # frequencies, but older versions of pandas do not support these as
+    # frequency strings.  Until xarray's minimum pandas version is 2.2 or above,
+    # we add logic to continue using the deprecated "M" and "Q" frequency
+    # strings in these circumstances.
+
+    # NOTE: other conversions ("h" -> "H", ..., "ns" -> "N") not required
+
+    # TODO: remove once requiring pandas >= 2.2
+    if not freq or Version(pd.__version__) >= Version("2.2"):
+        return freq
+
+    try:
+        freq_as_offset = to_offset(freq)
+    except ValueError:
+        # freq may be valid in pandas but not in xarray
+        return freq
+
+    if isinstance(freq_as_offset, MonthEnd) and "ME" in freq:
+        freq = freq.replace("ME", "M")
+    elif isinstance(freq_as_offset, QuarterEnd) and "QE" in freq:
+        freq = freq.replace("QE", "Q")
+    elif isinstance(freq_as_offset, YearBegin) and "YS" in freq:
+        freq = freq.replace("YS", "AS")
+    elif isinstance(freq_as_offset, YearEnd):
+        # testing for "Y" is required as this was valid in xarray 2023.11 - 2024.01
+        if "Y-" in freq:
+            # Check for and replace "Y-" instead of just "Y" to prevent
+            # corrupting anchored offsets that contain "Y" in the month
+            # abbreviation, e.g. "Y-MAY" -> "A-MAY".
+            freq = freq.replace("Y-", "A-")
+        elif "YE-" in freq:
+            freq = freq.replace("YE-", "A-")
+        elif "A-" not in freq and freq.endswith("Y"):
+            freq = freq.replace("Y", "A")
+        elif freq.endswith("YE"):
+            freq = freq.replace("YE", "A")
+
+    return freq
+
+
+def _legacy_to_new_freq(freq):
+    # to avoid internal deprecation warnings when freq is determined using pandas < 2.2
+
+    # TODO: remove once requiring pandas >= 2.2
+
+    if not freq or Version(pd.__version__) >= Version("2.2"):
+        return freq
+
+    try:
+        freq_as_offset = to_offset(freq, warn=False)
+    except ValueError:
+        # freq may be valid in pandas but not in xarray
+        return freq
+
+    if isinstance(freq_as_offset, MonthEnd) and "ME" not in freq:
+        freq = freq.replace("M", "ME")
+    elif isinstance(freq_as_offset, QuarterEnd) and "QE" not in freq:
+        freq = freq.replace("Q", "QE")
+    elif isinstance(freq_as_offset, YearBegin) and "YS" not in freq:
+        freq = freq.replace("AS", "YS")
+    elif isinstance(freq_as_offset, YearEnd):
+        if "A-" in freq:
+            # Check for and replace "A-" instead of just "A" to prevent
+            # corrupting anchored offsets that contain "Y" in the month
+            # abbreviation, e.g. "A-MAY" -> "YE-MAY".
+            freq = freq.replace("A-", "YE-")
+        elif "Y-" in freq:
+            freq = freq.replace("Y-", "YE-")
+        elif freq.endswith("A"):
+            # the "A-MAY" case is already handled above
+            freq = freq.replace("A", "YE")
+        elif "YE" not in freq and freq.endswith("Y"):
+            # the "Y-MAY" case is already handled above
+            freq = freq.replace("Y", "YE")
+    elif isinstance(freq_as_offset, Hour):
+        freq = freq.replace("H", "h")
+    elif isinstance(freq_as_offset, Minute):
+        freq = freq.replace("T", "min")
+    elif isinstance(freq_as_offset, Second):
+        freq = freq.replace("S", "s")
+    elif isinstance(freq_as_offset, Millisecond):
+        freq = freq.replace("L", "ms")
+    elif isinstance(freq_as_offset, Microsecond):
+        freq = freq.replace("U", "us")
+
+    return freq
+
+
 def date_range_like(source, calendar, use_cftime=None):
     """Generate a datetime array with the same frequency, start and end as
     another one, but in a different calendar.
@@ -1284,10 +1420,18 @@ def date_range_like(source, calendar, use_cftime=None):
             "`date_range_like` was unable to generate a range as the source frequency was not inferable."
         )
 
+    # TODO remove once requiring pandas >= 2.2
+    freq = _legacy_to_new_freq(freq)
+
     use_cftime = _should_cftime_be_used(source, calendar, use_cftime)
 
     source_start = source.values.min()
     source_end = source.values.max()
+
+    freq_as_offset = to_offset(freq)
+    if freq_as_offset.n < 0:
+        source_start, source_end = source_end, source_start
+
     if is_np_datetime_like(source.dtype):
         # We want to use datetime fields (datetime64 object don't have them)
         source_calendar = "standard"
@@ -1310,7 +1454,7 @@ def date_range_like(source, calendar, use_cftime=None):
 
     # For the cases where the source ends on the end of the month, we expect the same in the new calendar.
     if source_end.day == source_end.daysinmonth and isinstance(
-        to_offset(freq), (YearEnd, QuarterEnd, MonthEnd, Day)
+        freq_as_offset, (YearEnd, QuarterEnd, MonthEnd, Day)
     ):
         end = end.replace(day=end.daysinmonth)
 
