@@ -5,7 +5,7 @@ import operator
 from collections import defaultdict
 from collections.abc import Hashable, Iterable, Mapping
 from contextlib import suppress
-from typing import TYPE_CHECKING, Any, Callable, Generic, cast
+from typing import TYPE_CHECKING, Any, Callable, Final, Generic, TypeVar, cast, overload
 
 import numpy as np
 import pandas as pd
@@ -26,7 +26,13 @@ from xarray.core.variable import Variable, as_compatible_data, calculate_dimensi
 if TYPE_CHECKING:
     from xarray.core.dataarray import DataArray
     from xarray.core.dataset import Dataset
-    from xarray.core.types import JoinOptions, T_DataArray, T_Dataset, T_DuckArray
+    from xarray.core.types import (
+        Alignable,
+        JoinOptions,
+        T_DataArray,
+        T_Dataset,
+        T_DuckArray,
+    )
 
 
 def reindex_variables(
@@ -128,10 +134,10 @@ class Aligner(Generic[T_Alignable]):
         objects: Iterable[T_Alignable],
         join: str = "inner",
         indexes: Mapping[Any, Any] | None = None,
-        exclude_dims: Iterable = frozenset(),
+        exclude_dims: str | Iterable[Hashable] = frozenset(),
         exclude_vars: Iterable[Hashable] = frozenset(),
         method: str | None = None,
-        tolerance: int | float | Iterable[int | float] | None = None,
+        tolerance: float | Iterable[float] | str | None = None,
         copy: bool = True,
         fill_value: Any = dtypes.NA,
         sparse: bool = False,
@@ -318,7 +324,7 @@ class Aligner(Generic[T_Alignable]):
                     "- they may be used to reindex data along common dimensions"
                 )
 
-    def _need_reindex(self, dims, cmp_indexes) -> bool:
+    def _need_reindex(self, dim, cmp_indexes) -> bool:
         """Whether or not we need to reindex variables for a set of
         matching indexes.
 
@@ -334,14 +340,14 @@ class Aligner(Generic[T_Alignable]):
             return True
 
         unindexed_dims_sizes = {}
-        for dim in dims:
-            if dim in self.unindexed_dim_sizes:
-                sizes = self.unindexed_dim_sizes[dim]
+        for d in dim:
+            if d in self.unindexed_dim_sizes:
+                sizes = self.unindexed_dim_sizes[d]
                 if len(sizes) > 1:
                     # reindex if different sizes are found for unindexed dims
                     return True
                 else:
-                    unindexed_dims_sizes[dim] = next(iter(sizes))
+                    unindexed_dims_sizes[d] = next(iter(sizes))
 
         if unindexed_dims_sizes:
             indexed_dims_sizes = {}
@@ -350,8 +356,8 @@ class Aligner(Generic[T_Alignable]):
                 for var in index_vars.values():
                     indexed_dims_sizes.update(var.sizes)
 
-            for dim, size in unindexed_dims_sizes.items():
-                if indexed_dims_sizes.get(dim, -1) != size:
+            for d, size in unindexed_dims_sizes.items():
+                if indexed_dims_sizes.get(d, -1) != size:
                     # reindex if unindexed dimension size doesn't match
                     return True
 
@@ -576,12 +582,105 @@ class Aligner(Generic[T_Alignable]):
             self.reindex_all()
 
 
+T_Obj1 = TypeVar("T_Obj1", bound="Alignable")
+T_Obj2 = TypeVar("T_Obj2", bound="Alignable")
+T_Obj3 = TypeVar("T_Obj3", bound="Alignable")
+T_Obj4 = TypeVar("T_Obj4", bound="Alignable")
+T_Obj5 = TypeVar("T_Obj5", bound="Alignable")
+
+
+@overload
+def align(
+    obj1: T_Obj1,
+    /,
+    *,
+    join: JoinOptions = "inner",
+    copy: bool = True,
+    indexes=None,
+    exclude: str | Iterable[Hashable] = frozenset(),
+    fill_value=dtypes.NA,
+) -> tuple[T_Obj1]: ...
+
+
+@overload
+def align(
+    obj1: T_Obj1,
+    obj2: T_Obj2,
+    /,
+    *,
+    join: JoinOptions = "inner",
+    copy: bool = True,
+    indexes=None,
+    exclude: str | Iterable[Hashable] = frozenset(),
+    fill_value=dtypes.NA,
+) -> tuple[T_Obj1, T_Obj2]: ...
+
+
+@overload
+def align(
+    obj1: T_Obj1,
+    obj2: T_Obj2,
+    obj3: T_Obj3,
+    /,
+    *,
+    join: JoinOptions = "inner",
+    copy: bool = True,
+    indexes=None,
+    exclude: str | Iterable[Hashable] = frozenset(),
+    fill_value=dtypes.NA,
+) -> tuple[T_Obj1, T_Obj2, T_Obj3]: ...
+
+
+@overload
+def align(
+    obj1: T_Obj1,
+    obj2: T_Obj2,
+    obj3: T_Obj3,
+    obj4: T_Obj4,
+    /,
+    *,
+    join: JoinOptions = "inner",
+    copy: bool = True,
+    indexes=None,
+    exclude: str | Iterable[Hashable] = frozenset(),
+    fill_value=dtypes.NA,
+) -> tuple[T_Obj1, T_Obj2, T_Obj3, T_Obj4]: ...
+
+
+@overload
+def align(
+    obj1: T_Obj1,
+    obj2: T_Obj2,
+    obj3: T_Obj3,
+    obj4: T_Obj4,
+    obj5: T_Obj5,
+    /,
+    *,
+    join: JoinOptions = "inner",
+    copy: bool = True,
+    indexes=None,
+    exclude: str | Iterable[Hashable] = frozenset(),
+    fill_value=dtypes.NA,
+) -> tuple[T_Obj1, T_Obj2, T_Obj3, T_Obj4, T_Obj5]: ...
+
+
+@overload
 def align(
     *objects: T_Alignable,
     join: JoinOptions = "inner",
     copy: bool = True,
     indexes=None,
-    exclude=frozenset(),
+    exclude: str | Iterable[Hashable] = frozenset(),
+    fill_value=dtypes.NA,
+) -> tuple[T_Alignable, ...]: ...
+
+
+def align(
+    *objects: T_Alignable,
+    join: JoinOptions = "inner",
+    copy: bool = True,
+    indexes=None,
+    exclude: str | Iterable[Hashable] = frozenset(),
     fill_value=dtypes.NA,
 ) -> tuple[T_Alignable, ...]:
     """
@@ -620,7 +719,7 @@ def align(
     indexes : dict-like, optional
         Any indexes explicitly provided with the `indexes` argument should be
         used in preference to the aligned indexes.
-    exclude : sequence of str, optional
+    exclude : str, iterable of hashable or None, optional
         Dimensions that must be excluded from alignment
     fill_value : scalar or dict-like, optional
         Value to use for newly missing values. If a dict-like, maps
@@ -653,102 +752,102 @@ def align(
     ... )
 
     >>> x
-    <xarray.DataArray (lat: 2, lon: 2)>
+    <xarray.DataArray (lat: 2, lon: 2)> Size: 32B
     array([[25, 35],
            [10, 24]])
     Coordinates:
-      * lat      (lat) float64 35.0 40.0
-      * lon      (lon) float64 100.0 120.0
+      * lat      (lat) float64 16B 35.0 40.0
+      * lon      (lon) float64 16B 100.0 120.0
 
     >>> y
-    <xarray.DataArray (lat: 2, lon: 2)>
+    <xarray.DataArray (lat: 2, lon: 2)> Size: 32B
     array([[20,  5],
            [ 7, 13]])
     Coordinates:
-      * lat      (lat) float64 35.0 42.0
-      * lon      (lon) float64 100.0 120.0
+      * lat      (lat) float64 16B 35.0 42.0
+      * lon      (lon) float64 16B 100.0 120.0
 
     >>> a, b = xr.align(x, y)
     >>> a
-    <xarray.DataArray (lat: 1, lon: 2)>
+    <xarray.DataArray (lat: 1, lon: 2)> Size: 16B
     array([[25, 35]])
     Coordinates:
-      * lat      (lat) float64 35.0
-      * lon      (lon) float64 100.0 120.0
+      * lat      (lat) float64 8B 35.0
+      * lon      (lon) float64 16B 100.0 120.0
     >>> b
-    <xarray.DataArray (lat: 1, lon: 2)>
+    <xarray.DataArray (lat: 1, lon: 2)> Size: 16B
     array([[20,  5]])
     Coordinates:
-      * lat      (lat) float64 35.0
-      * lon      (lon) float64 100.0 120.0
+      * lat      (lat) float64 8B 35.0
+      * lon      (lon) float64 16B 100.0 120.0
 
     >>> a, b = xr.align(x, y, join="outer")
     >>> a
-    <xarray.DataArray (lat: 3, lon: 2)>
+    <xarray.DataArray (lat: 3, lon: 2)> Size: 48B
     array([[25., 35.],
            [10., 24.],
            [nan, nan]])
     Coordinates:
-      * lat      (lat) float64 35.0 40.0 42.0
-      * lon      (lon) float64 100.0 120.0
+      * lat      (lat) float64 24B 35.0 40.0 42.0
+      * lon      (lon) float64 16B 100.0 120.0
     >>> b
-    <xarray.DataArray (lat: 3, lon: 2)>
+    <xarray.DataArray (lat: 3, lon: 2)> Size: 48B
     array([[20.,  5.],
            [nan, nan],
            [ 7., 13.]])
     Coordinates:
-      * lat      (lat) float64 35.0 40.0 42.0
-      * lon      (lon) float64 100.0 120.0
+      * lat      (lat) float64 24B 35.0 40.0 42.0
+      * lon      (lon) float64 16B 100.0 120.0
 
     >>> a, b = xr.align(x, y, join="outer", fill_value=-999)
     >>> a
-    <xarray.DataArray (lat: 3, lon: 2)>
+    <xarray.DataArray (lat: 3, lon: 2)> Size: 48B
     array([[  25,   35],
            [  10,   24],
            [-999, -999]])
     Coordinates:
-      * lat      (lat) float64 35.0 40.0 42.0
-      * lon      (lon) float64 100.0 120.0
+      * lat      (lat) float64 24B 35.0 40.0 42.0
+      * lon      (lon) float64 16B 100.0 120.0
     >>> b
-    <xarray.DataArray (lat: 3, lon: 2)>
+    <xarray.DataArray (lat: 3, lon: 2)> Size: 48B
     array([[  20,    5],
            [-999, -999],
            [   7,   13]])
     Coordinates:
-      * lat      (lat) float64 35.0 40.0 42.0
-      * lon      (lon) float64 100.0 120.0
+      * lat      (lat) float64 24B 35.0 40.0 42.0
+      * lon      (lon) float64 16B 100.0 120.0
 
     >>> a, b = xr.align(x, y, join="left")
     >>> a
-    <xarray.DataArray (lat: 2, lon: 2)>
+    <xarray.DataArray (lat: 2, lon: 2)> Size: 32B
     array([[25, 35],
            [10, 24]])
     Coordinates:
-      * lat      (lat) float64 35.0 40.0
-      * lon      (lon) float64 100.0 120.0
+      * lat      (lat) float64 16B 35.0 40.0
+      * lon      (lon) float64 16B 100.0 120.0
     >>> b
-    <xarray.DataArray (lat: 2, lon: 2)>
+    <xarray.DataArray (lat: 2, lon: 2)> Size: 32B
     array([[20.,  5.],
            [nan, nan]])
     Coordinates:
-      * lat      (lat) float64 35.0 40.0
-      * lon      (lon) float64 100.0 120.0
+      * lat      (lat) float64 16B 35.0 40.0
+      * lon      (lon) float64 16B 100.0 120.0
 
     >>> a, b = xr.align(x, y, join="right")
     >>> a
-    <xarray.DataArray (lat: 2, lon: 2)>
+    <xarray.DataArray (lat: 2, lon: 2)> Size: 32B
     array([[25., 35.],
            [nan, nan]])
     Coordinates:
-      * lat      (lat) float64 35.0 42.0
-      * lon      (lon) float64 100.0 120.0
+      * lat      (lat) float64 16B 35.0 42.0
+      * lon      (lon) float64 16B 100.0 120.0
     >>> b
-    <xarray.DataArray (lat: 2, lon: 2)>
+    <xarray.DataArray (lat: 2, lon: 2)> Size: 32B
     array([[20,  5],
            [ 7, 13]])
     Coordinates:
-      * lat      (lat) float64 35.0 42.0
-      * lon      (lon) float64 100.0 120.0
+      * lat      (lat) float64 16B 35.0 42.0
+      * lon      (lon) float64 16B 100.0 120.0
 
     >>> a, b = xr.align(x, y, join="exact")
     Traceback (most recent call last):
@@ -757,19 +856,19 @@ def align(
 
     >>> a, b = xr.align(x, y, join="override")
     >>> a
-    <xarray.DataArray (lat: 2, lon: 2)>
+    <xarray.DataArray (lat: 2, lon: 2)> Size: 32B
     array([[25, 35],
            [10, 24]])
     Coordinates:
-      * lat      (lat) float64 35.0 40.0
-      * lon      (lon) float64 100.0 120.0
+      * lat      (lat) float64 16B 35.0 40.0
+      * lon      (lon) float64 16B 100.0 120.0
     >>> b
-    <xarray.DataArray (lat: 2, lon: 2)>
+    <xarray.DataArray (lat: 2, lon: 2)> Size: 32B
     array([[20,  5],
            [ 7, 13]])
     Coordinates:
-      * lat      (lat) float64 35.0 40.0
-      * lon      (lon) float64 100.0 120.0
+      * lat      (lat) float64 16B 35.0 40.0
+      * lon      (lon) float64 16B 100.0 120.0
 
     """
     aligner = Aligner(
@@ -787,12 +886,12 @@ def align(
 def deep_align(
     objects: Iterable[Any],
     join: JoinOptions = "inner",
-    copy=True,
+    copy: bool = True,
     indexes=None,
-    exclude=frozenset(),
-    raise_on_invalid=True,
+    exclude: str | Iterable[Hashable] = frozenset(),
+    raise_on_invalid: bool = True,
     fill_value=dtypes.NA,
-):
+) -> list[Any]:
     """Align objects for merging, recursing into dictionary values.
 
     This function is not public API.
@@ -807,12 +906,12 @@ def deep_align(
     def is_alignable(obj):
         return isinstance(obj, (Coordinates, DataArray, Dataset))
 
-    positions = []
-    keys = []
-    out = []
-    targets = []
-    no_key = object()
-    not_replaced = object()
+    positions: list[int] = []
+    keys: list[type[object] | Hashable] = []
+    out: list[Any] = []
+    targets: list[Alignable] = []
+    no_key: Final = object()
+    not_replaced: Final = object()
     for position, variables in enumerate(objects):
         if is_alignable(variables):
             positions.append(position)
@@ -857,7 +956,7 @@ def deep_align(
         if key is no_key:
             out[position] = aligned_obj
         else:
-            out[position][key] = aligned_obj  # type: ignore[index]  # maybe someone can fix this?
+            out[position][key] = aligned_obj
 
     return out
 
@@ -866,7 +965,7 @@ def reindex(
     obj: T_Alignable,
     indexers: Mapping[Any, Any],
     method: str | None = None,
-    tolerance: int | float | Iterable[int | float] | None = None,
+    tolerance: float | Iterable[float] | str | None = None,
     copy: bool = True,
     fill_value: Any = dtypes.NA,
     sparse: bool = False,
@@ -905,7 +1004,7 @@ def reindex_like(
     obj: T_Alignable,
     other: Dataset | DataArray,
     method: str | None = None,
-    tolerance: int | float | Iterable[int | float] | None = None,
+    tolerance: float | Iterable[float] | str | None = None,
     copy: bool = True,
     fill_value: Any = dtypes.NA,
 ) -> T_Alignable:
@@ -988,9 +1087,63 @@ def _broadcast_helper(
         raise ValueError("all input must be Dataset or DataArray objects")
 
 
-# TODO: this typing is too restrictive since it cannot deal with mixed
-# DataArray and Dataset types...? Is this a problem?
-def broadcast(*args: T_Alignable, exclude=None) -> tuple[T_Alignable, ...]:
+@overload
+def broadcast(
+    obj1: T_Obj1, /, *, exclude: str | Iterable[Hashable] | None = None
+) -> tuple[T_Obj1]: ...
+
+
+@overload
+def broadcast(
+    obj1: T_Obj1, obj2: T_Obj2, /, *, exclude: str | Iterable[Hashable] | None = None
+) -> tuple[T_Obj1, T_Obj2]: ...
+
+
+@overload
+def broadcast(
+    obj1: T_Obj1,
+    obj2: T_Obj2,
+    obj3: T_Obj3,
+    /,
+    *,
+    exclude: str | Iterable[Hashable] | None = None,
+) -> tuple[T_Obj1, T_Obj2, T_Obj3]: ...
+
+
+@overload
+def broadcast(
+    obj1: T_Obj1,
+    obj2: T_Obj2,
+    obj3: T_Obj3,
+    obj4: T_Obj4,
+    /,
+    *,
+    exclude: str | Iterable[Hashable] | None = None,
+) -> tuple[T_Obj1, T_Obj2, T_Obj3, T_Obj4]: ...
+
+
+@overload
+def broadcast(
+    obj1: T_Obj1,
+    obj2: T_Obj2,
+    obj3: T_Obj3,
+    obj4: T_Obj4,
+    obj5: T_Obj5,
+    /,
+    *,
+    exclude: str | Iterable[Hashable] | None = None,
+) -> tuple[T_Obj1, T_Obj2, T_Obj3, T_Obj4, T_Obj5]: ...
+
+
+@overload
+def broadcast(
+    *args: T_Alignable, exclude: str | Iterable[Hashable] | None = None
+) -> tuple[T_Alignable, ...]: ...
+
+
+def broadcast(
+    *args: T_Alignable, exclude: str | Iterable[Hashable] | None = None
+) -> tuple[T_Alignable, ...]:
     """Explicitly broadcast any number of DataArray or Dataset objects against
     one another.
 
@@ -1004,7 +1157,7 @@ def broadcast(*args: T_Alignable, exclude=None) -> tuple[T_Alignable, ...]:
     ----------
     *args : DataArray or Dataset
         Arrays to broadcast against each other.
-    exclude : sequence of str, optional
+    exclude : str, iterable of hashable or None, optional
         Dimensions that must not be broadcasted
 
     Returns
@@ -1020,22 +1173,22 @@ def broadcast(*args: T_Alignable, exclude=None) -> tuple[T_Alignable, ...]:
     >>> a = xr.DataArray([1, 2, 3], dims="x")
     >>> b = xr.DataArray([5, 6], dims="y")
     >>> a
-    <xarray.DataArray (x: 3)>
+    <xarray.DataArray (x: 3)> Size: 24B
     array([1, 2, 3])
     Dimensions without coordinates: x
     >>> b
-    <xarray.DataArray (y: 2)>
+    <xarray.DataArray (y: 2)> Size: 16B
     array([5, 6])
     Dimensions without coordinates: y
     >>> a2, b2 = xr.broadcast(a, b)
     >>> a2
-    <xarray.DataArray (x: 3, y: 2)>
+    <xarray.DataArray (x: 3, y: 2)> Size: 48B
     array([[1, 1],
            [2, 2],
            [3, 3]])
     Dimensions without coordinates: x, y
     >>> b2
-    <xarray.DataArray (x: 3, y: 2)>
+    <xarray.DataArray (x: 3, y: 2)> Size: 48B
     array([[5, 6],
            [5, 6],
            [5, 6]])
@@ -1046,12 +1199,12 @@ def broadcast(*args: T_Alignable, exclude=None) -> tuple[T_Alignable, ...]:
     >>> ds = xr.Dataset({"a": a, "b": b})
     >>> (ds2,) = xr.broadcast(ds)  # use tuple unpacking to extract one dataset
     >>> ds2
-    <xarray.Dataset>
+    <xarray.Dataset> Size: 96B
     Dimensions:  (x: 3, y: 2)
     Dimensions without coordinates: x, y
     Data variables:
-        a        (x, y) int64 1 1 2 2 3 3
-        b        (x, y) int64 5 6 5 6 5 6
+        a        (x, y) int64 48B 1 1 2 2 3 3
+        b        (x, y) int64 48B 5 6 5 6 5 6
     """
 
     if exclude is None:
