@@ -13,12 +13,18 @@ import pandas as pd
 from xarray.core import utils
 from xarray.core.common import _contains_datetime_like_objects, ones_like
 from xarray.core.computation import apply_ufunc
-from xarray.core.duck_array_ops import datetime_to_numeric, push, timedelta_to_numeric
-from xarray.core.options import OPTIONS, _get_keep_attrs
-from xarray.core.parallelcompat import get_chunked_array_type, is_chunked_array
+from xarray.core.duck_array_ops import (
+    datetime_to_numeric,
+    push,
+    reshape,
+    timedelta_to_numeric,
+)
+from xarray.core.options import _get_keep_attrs
 from xarray.core.types import Interp1dOptions, InterpOptions
 from xarray.core.utils import OrderedSet, is_scalar
 from xarray.core.variable import Variable, broadcast_variables
+from xarray.namedarray.parallelcompat import get_chunked_array_type
+from xarray.namedarray.pycompat import is_chunked_array
 
 if TYPE_CHECKING:
     from xarray.core.dataarray import DataArray
@@ -309,7 +315,9 @@ def interp_na(
     use_coordinate: bool | str = True,
     method: InterpOptions = "linear",
     limit: int | None = None,
-    max_gap: int | float | str | pd.Timedelta | np.timedelta64 | dt.timedelta = None,
+    max_gap: (
+        int | float | str | pd.Timedelta | np.timedelta64 | dt.timedelta | None
+    ) = None,
     keep_attrs: bool | None = None,
     **kwargs,
 ):
@@ -413,11 +421,6 @@ def _bfill(arr, n=None, axis=-1):
 
 def ffill(arr, dim=None, limit=None):
     """forward fill missing values"""
-    if not OPTIONS["use_bottleneck"]:
-        raise RuntimeError(
-            "ffill requires bottleneck to be enabled."
-            " Call `xr.set_options(use_bottleneck=True)` to enable it."
-        )
 
     axis = arr.get_axis_num(dim)
 
@@ -436,11 +439,6 @@ def ffill(arr, dim=None, limit=None):
 
 def bfill(arr, dim=None, limit=None):
     """backfill missing values"""
-    if not OPTIONS["use_bottleneck"]:
-        raise RuntimeError(
-            "bfill requires bottleneck to be enabled."
-            " Call `xr.set_options(use_bottleneck=True)` to enable it."
-        )
 
     axis = arr.get_axis_num(dim)
 
@@ -474,9 +472,9 @@ def _get_interpolator(
 
     returns interpolator class and keyword arguments for the class
     """
-    interp_class: type[NumpyInterpolator] | type[ScipyInterpolator] | type[
-        SplineInterpolator
-    ]
+    interp_class: (
+        type[NumpyInterpolator] | type[ScipyInterpolator] | type[SplineInterpolator]
+    )
 
     interp1d_methods = get_args(Interp1dOptions)
     valid_methods = tuple(vv for v in get_args(InterpOptions) for vv in get_args(v))
@@ -557,11 +555,11 @@ def _localize(var, indexes_coords):
     """
     indexes = {}
     for dim, [x, new_x] in indexes_coords.items():
-        minval = np.nanmin(new_x.values)
-        maxval = np.nanmax(new_x.values)
+        new_x_loaded = new_x.values
+        minval = np.nanmin(new_x_loaded)
+        maxval = np.nanmax(new_x_loaded)
         index = x.to_index()
-        imin = index.get_indexer([minval], method="nearest").item()
-        imax = index.get_indexer([maxval], method="nearest").item()
+        imin, imax = index.get_indexer([minval, maxval], method="nearest")
         indexes[dim] = slice(max(imin - 2, 0), imax + 2)
         indexes_coords[dim] = (x[indexes[dim]], new_x)
     return var.isel(**indexes), indexes_coords
@@ -678,7 +676,7 @@ def interp_func(var, x, new_x, method: InterpOptions, kwargs):
 
     Notes
     -----
-    This requiers scipy installed.
+    This requires scipy installed.
 
     See Also
     --------
@@ -724,7 +722,7 @@ def interp_func(var, x, new_x, method: InterpOptions, kwargs):
             for i in range(new_x[0].ndim)
         }
 
-        # if useful, re-use localize for each chunk of new_x
+        # if useful, reuse localize for each chunk of new_x
         localize = (method in ["linear", "nearest"]) and new_x0_chunks_is_not_none
 
         # scipy.interpolate.interp1d always forces to float.
@@ -758,7 +756,7 @@ def _interp1d(var, x, new_x, func, kwargs):
     x, new_x = x[0], new_x[0]
     rslt = func(x, var, assume_sorted=True, **kwargs)(np.ravel(new_x))
     if new_x.ndim > 1:
-        return rslt.reshape(var.shape[:-1] + new_x.shape)
+        return reshape(rslt, (var.shape[:-1] + new_x.shape))
     if new_x.ndim == 0:
         return rslt[..., -1]
     return rslt
@@ -777,7 +775,7 @@ def _interpnd(var, x, new_x, func, kwargs):
     rslt = func(x, var, xi, **kwargs)
     # move back the interpolation axes to the last position
     rslt = rslt.transpose(range(-rslt.ndim + 1, 1))
-    return rslt.reshape(rslt.shape[:-1] + new_x[0].shape)
+    return reshape(rslt, rslt.shape[:-1] + new_x[0].shape)
 
 
 def _chunked_aware_interpnd(var, *coords, interp_func, interp_kwargs, localize=True):
